@@ -151,6 +151,12 @@ typedef enum
   META_FOCUS_MODE_MOUSE
 } MetaFocusMode;
 
+typedef enum
+{
+  META_FOCUS_NEW_WINDOWS_SMART,
+  META_FOCUS_NEW_WINDOWS_STRICT
+} MetaFocusNewWindows;
+
 typedef enum
 {
   META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_SHADE,
