@@ -42,6 +42,7 @@
  */
 #define KEY_MOUSE_BUTTON_MODS "/apps/metacity/general/mouse_button_modifier"
 #define KEY_FOCUS_MODE "/apps/metacity/general/focus_mode"
+#define KEY_FOCUS_NEW_WINDOWS "/apps/metacity/general/focus_new_windows"
 #define KEY_RAISE_ON_CLICK "/apps/metacity/general/raise_on_click"
 #define KEY_ACTION_DOUBLE_CLICK_TITLEBAR "/apps/metacity/general/action_double_click_titlebar"
 #define KEY_AUTO_RAISE "/apps/metacity/general/auto_raise"
@@ -82,6 +83,7 @@ static gboolean use_system_font = FALSE;
 static PangoFontDescription *titlebar_font = NULL;
 static MetaVirtualModifier mouse_button_mods = Mod1Mask;
 static MetaFocusMode focus_mode = META_FOCUS_MODE_CLICK;
+static MetaFocusNewWindows focus_new_windows = META_FOCUS_NEW_WINDOWS_SMART;
 static gboolean raise_on_click = TRUE;
 static char* current_theme = NULL;
 static int num_workspaces = 4;
@@ -126,6 +128,7 @@ static gboolean update_use_system_font    (gboolean    value);
 static gboolean update_titlebar_font      (const char *value);
 static gboolean update_mouse_button_mods  (const char *value);
 static gboolean update_focus_mode         (const char *value);
+static gboolean update_focus_new_windows  (const char *value);
 static gboolean update_raise_on_click     (gboolean    value);
 static gboolean update_theme              (const char *value);
 static gboolean update_visual_bell        (gboolean v1, gboolean v2);
@@ -334,6 +337,8 @@ meta_prefs_init (void)
   char *str_val;
   int int_val;
   gboolean bool_val, bool_val_2;
+  gboolean update_visual;
+  gboolean update_audible;
   
   if (default_client != NULL)
     return;
@@ -373,6 +378,12 @@ meta_prefs_init (void)
   update_focus_mode (str_val);
   g_free (str_val);  
 
+  str_val = gconf_client_get_string (default_client, KEY_FOCUS_NEW_WINDOWS,
+                                     &err);
+  cleanup_error (&err);
+  update_focus_new_windows (str_val);
+  g_free (str_val);  
+
   if (get_bool (KEY_RAISE_ON_CLICK, &bool_val))
     update_raise_on_click (bool_val);
   
@@ -434,11 +445,14 @@ meta_prefs_init (void)
   cleanup_error (&err);
   update_button_layout (str_val);
   g_free (str_val);
-  
-  if (get_bool (KEY_VISUAL_BELL,  &bool_val) ||
-      get_bool (KEY_AUDIBLE_BELL, &bool_val_2))
-    update_visual_bell (bool_val, bool_val_2);
 
+  bool_val = provide_visual_bell;
+  bool_val_2 = bell_is_audible;
+  update_visual = get_bool (KEY_VISUAL_BELL,  &bool_val);
+  update_audible = get_bool (KEY_AUDIBLE_BELL, &bool_val_2);
+  if (update_visual || update_audible)
+    update_visual_bell (bool_val, bool_val_2);
+      
   str_val = gconf_client_get_string (default_client, KEY_VISUAL_BELL_TYPE,
                                      &err);
   cleanup_error (&err);
@@ -513,26 +527,6 @@ meta_prefs_init (void)
 }
 
 #ifdef HAVE_GCONF
-/* from eel */
-static gboolean
-str_has_prefix (const char *haystack, const char *needle)
-{
-  const char *h, *n;
-  
-  /* Eat one character at a time. */
-  h = haystack == NULL ? "" : haystack;
-  n = needle == NULL ? "" : needle;
-  do
-    {
-      if (*n == '\0') 
-        return TRUE;
-      if (*h == '\0')
-        return FALSE;
-    }
-  while (*h++ == *n++);
-
-  return FALSE;
-}
 
 static void
 change_notify (GConfClient    *client,
@@ -578,6 +572,22 @@ change_notify (GConfClient    *client,
       if (update_focus_mode (str))
         queue_changed (META_PREF_FOCUS_MODE);
     }
+  else if (strcmp (key, KEY_FOCUS_NEW_WINDOWS) == 0)
+    {
+      const char *str;
+
+      if (value && value->type != GCONF_VALUE_STRING)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                        KEY_FOCUS_NEW_WINDOWS);
+          goto out;
+        }
+      
+      str = value ? gconf_value_get_string (value) : NULL;
+
+      if (update_focus_new_windows (str))
+        queue_changed (META_PREF_FOCUS_NEW_WINDOWS);
+    }
   else if (strcmp (key, KEY_RAISE_ON_CLICK) == 0)
     {
       gboolean b;
@@ -694,7 +704,7 @@ change_notify (GConfClient    *client,
       if (update_disable_workarounds (b))
         queue_changed (META_PREF_DISABLE_WORKAROUNDS);
     }
-  else if (str_has_prefix (key, KEY_WINDOW_BINDINGS_PREFIX))
+  else if (g_str_has_prefix (key, KEY_WINDOW_BINDINGS_PREFIX))
     {
       const char *str;
 
@@ -710,7 +720,7 @@ change_notify (GConfClient    *client,
       if (update_window_binding (key, str))
         queue_changed (META_PREF_WINDOW_KEYBINDINGS);
     }
-  else if (str_has_prefix (key, KEY_SCREEN_BINDINGS_PREFIX))
+  else if (g_str_has_prefix (key, KEY_SCREEN_BINDINGS_PREFIX))
     {
       const char *str;
 
@@ -775,7 +785,7 @@ change_notify (GConfClient    *client,
         queue_changed (META_PREF_AUTO_RAISE_DELAY);
     
     }
-  else if (str_has_prefix (key, KEY_COMMAND_PREFIX))
+  else if (g_str_has_prefix (key, KEY_COMMAND_PREFIX))
     {
       const char *str;
 
@@ -807,7 +817,7 @@ change_notify (GConfClient    *client,
       if (update_terminal_command (str))
         queue_changed (META_PREF_TERMINAL_COMMAND);
     }
-  else if (str_has_prefix (key, KEY_WORKSPACE_NAME_PREFIX))
+  else if (g_str_has_prefix (key, KEY_WORKSPACE_NAME_PREFIX))
     {
       const char *str;
 
@@ -1015,6 +1025,27 @@ update_focus_mode (const char *value)
 }
 #endif /* HAVE_GCONF */
 
+#ifdef HAVE_GCONF
+static gboolean
+update_focus_new_windows (const char *value)
+{
+  MetaFocusNewWindows old_mode = focus_new_windows;
+  
+  if (value != NULL)
+    {
+      if (g_ascii_strcasecmp (value, "smart") == 0)
+        focus_new_windows = META_FOCUS_NEW_WINDOWS_SMART;
+      else if (g_ascii_strcasecmp (value, "strict") == 0)
+        focus_new_windows = META_FOCUS_NEW_WINDOWS_STRICT;
+      else
+        meta_warning (_("GConf key '%s' is set to an invalid value\n"),
+                      KEY_FOCUS_NEW_WINDOWS);
+    }
+
+  return (old_mode != focus_new_windows);
+}
+#endif /* HAVE_GCONF */
+
 #ifdef HAVE_GCONF
 static gboolean
 update_raise_on_click (gboolean value)
@@ -1073,6 +1104,12 @@ meta_prefs_get_focus_mode (void)
   return focus_mode;
 }
 
+MetaFocusNewWindows
+meta_prefs_get_focus_new_windows (void)
+{
+  return focus_new_windows;
+}
+
 gboolean
 meta_prefs_get_raise_on_click (void)
 {
@@ -1477,6 +1514,10 @@ action_double_click_titlebar_from_string (const char *str)
     return META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_SHADE;
   else if (strcmp (str, "toggle_maximize") == 0)
     return META_ACTION_DOUBLE_CLICK_TITLEBAR_TOGGLE_MAXIMIZE;
+  else if (strcmp (str, "minimize") == 0)
+    return META_ACTION_DOUBLE_CLICK_TITLEBAR_MINIMIZE;
+  else if (strcmp (str, "none") == 0)
+    return META_ACTION_DOUBLE_CLICK_TITLEBAR_NONE;
   else
     return META_ACTION_DOUBLE_CLICK_TITLEBAR_LAST;
 }
@@ -1564,6 +1605,9 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_FOCUS_MODE:
       return "FOCUS_MODE";
 
+    case META_PREF_FOCUS_NEW_WINDOWS:
+      return "FOCUS_NEW_WINDOWS";
+
     case META_PREF_RAISE_ON_CLICK:
       return "RAISE_ON_CLICK";
       
@@ -1995,8 +2039,70 @@ update_binding (MetaKeyPref *binding,
                       "Failed to parse new gconf value\n");
           meta_warning (_("\"%s\" found in configuration database is not a valid value for keybinding \"%s\"\n"),
                         value, binding->name);
+
+          return FALSE;
         }
     }
+
+  /* Bug 329676: Bindings which can be shifted must not have no modifiers,
+   * nor only SHIFT as a modifier.
+   */
+
+  if (binding->add_shift &&
+      0 != keysym &&
+      (META_VIRTUAL_SHIFT_MASK == mods || 0 == mods))
+    {
+      gchar *old_setting;
+      gchar *key;
+      GError *err = NULL;
+      
+      meta_warning ("Cannot bind \"%s\" to %s: it needs a modifier "
+                    "such as Ctrl or Alt.\n",
+                    binding->name,
+                    value);
+
+      old_setting = meta_ui_accelerator_name(
+                      binding->keysym,
+                      binding->modifiers);
+      
+      if (!strcmp(old_setting, value))
+        {
+          /* We were about to set it to the same value
+           * that it had originally! This must be caused
+           * by getting an invalid string back from
+           * meta_ui_accelerator_name. Bail out now
+           * so we don't get into an infinite loop.
+           */
+           return TRUE;
+        }
+
+      meta_warning ("Reverting \"%s\" to %s.\n",
+                    binding->name,
+                    old_setting);
+
+      key = g_strconcat (KEY_SCREEN_BINDINGS_PREFIX, "/",
+                         binding->name, NULL);
+      
+      gconf_client_set_string (gconf_client_get_default (),
+                               key, old_setting, &err);
+
+      if (err)
+        {
+          meta_warning ("Error while reverting keybinding: %s\n",
+                        err->message);
+          g_error_free (err);
+          err = NULL;
+        }
+      
+      g_free (old_setting);
+      g_free (key);
+
+      /* The call to gconf_client_set_string() will cause this function
+       * to be called again with the new value, so there's no need to
+       * carry on.
+       */
+      return TRUE;
+    }
   
   changed = FALSE;
   if (keysym != binding->keysym ||
